@@ -859,6 +859,10 @@ dbFindIdsUniqueTrials <- function(
       include3rdcountrytrials = include3rdcountrytrials)
   }
 
+  # keep only euctr
+  listofEUCTRids <- listofEUCTRids[
+    !is.na(listofEUCTRids["a2_eudract_number"]), ]
+
   # 3. get ctgov records
   listofCTGOVids <- try(suppressMessages(suppressWarnings(
     dbGetFieldsIntoDf(fields = c(
@@ -935,7 +939,6 @@ dbFindIdsUniqueTrials <- function(
       }
       #
       # c.2 - ctgov in euctr a52_... (id_info corresponds to index 2)
-      # TODO deleteme
       dupes.c2 <- sapply(
         listofCTGOVids[["id_info"]],
         function(x) any(
@@ -968,7 +971,7 @@ dbFindIdsUniqueTrials <- function(
             listofEUCTRids[["a41_sponsors_protocol_code_number"]]))
       #
       if (verbose) {
-        message(" - ", sum(dupes.d2),
+        message(" - ", sum(dupes.e2),
                 " CTGOV secondary_id / nct_alias / org_study_id in",
                 " EUCTR a41_sponsors_protocol_...")}
       #
@@ -988,7 +991,7 @@ dbFindIdsUniqueTrials <- function(
     #
     if (preferregister == "CTGOV") {
       #
-      # a.1 - euctr in ctgov (id_info corresponds to index 2)
+      # a.1 - euctr in ctgov
       dupes.a1 <- listofEUCTRids[["a2_eudract_number"]] %in% sub(
         ".*([0-9]{4}-[0-9]{6}-[0-9]{2}).*", # e.g. "EUDRACT-2004-000242-20"
         "\\1", unlist(listofCTGOVids[["id_info"]]))
@@ -998,16 +1001,16 @@ dbFindIdsUniqueTrials <- function(
                 " EUCTR _id in CTGOV secondary_id / nct_alias / org_study_id")
       }
       #
-      # b.1 - euctr in ctgov (_id corresponds to index 1)
+      # b.1 - euctr in ctgov
       dupes.b1 <- listofEUCTRids[[
         "a52_us_nct_clinicaltrialsgov_registry_number"]] %in% listofCTGOVids[["_id"]]
       #
       if (verbose) {
         message(" - ", sum(dupes.b1),
-                           " EUCTR a52_us_nct_... in CTGOV _id (nct)")
+                " EUCTR a52_us_nct_... in CTGOV _id (nct)")
       }
       #
-      # c.1 - euctr in ctgov (id_info corresponds to index 2)
+      # c.1 - euctr in ctgov
       dupes.c1 <- listofEUCTRids[[
         "a52_us_nct_clinicaltrialsgov_registry_number"]] %in%
         unlist(listofCTGOVids[["id_info"]])
@@ -1019,7 +1022,7 @@ dbFindIdsUniqueTrials <- function(
           " CTOGV secondary_id / nct_alias / org_study_id")
       }
       #
-      # d.1 - euctr in ctgov (id_info corresponds to index 2)
+      # d.1 - euctr in ctgov
       dupes.d1 <- listofEUCTRids[[
         "a51_isrctn_international_standard_randomised_controlled_trial_number"]] %in%
         unlist(listofCTGOVids[["id_info"]])
@@ -1031,7 +1034,7 @@ dbFindIdsUniqueTrials <- function(
           " in CTOGV secondary_id / nct_alias / org_study_id")
       }
       #
-      # e.1 - euctr in ctgov (id_info corresponds to index 2)
+      # e.1 - euctr in ctgov
       dupes.e1 <- listofEUCTRids[["a41_sponsors_protocol_code_number"]] %in%
         unlist(listofCTGOVids[["id_info"]])
       #
@@ -1341,6 +1344,363 @@ dbGetFieldsIntoDf <- function(fields = "",
 # dbGetFieldsIntoDf
 
 
+#' Extract information of interest (e.g., endpoint)
+#' from long data frame of protocol- or result-related
+#' trial information as returned from \link{dfTrials2Long}
+#'
+#' @param df A data frame with 5 columns (trial_id,
+#'  main_id, sub_id, name, value) as returned from
+#'  \link{dfTrials2Long}
+#'
+#' @param valuename A character string for the name of the variable
+#'  from which to extract information for the variable of interest
+#'
+#' @param wherename A character string to identify the variable
+#'  of interest
+#'
+#' @param wherevalue A character string with the value of interest
+#'  for the variable of interest
+#'
+#' @return A data frame with columns trial_id, main_id, sub_id,
+#'  name, value that only includes the values of interest
+#'
+#' @export
+#' @examples
+#' \dontrun{
+#' db <- nodbi::src_sqlite(
+#'   collection = "my_collection"
+#' )
+#' df <- ctrdata::dbGetFieldsIntoDf(
+#' fields = c(
+#'   # ctgov - typical results fields
+#'   "clinical_results.baseline.analyzed_list.analyzed.count_list.count",
+#'   "clinical_results.baseline.group_list.group",
+#'   "clinical_results.baseline.analyzed_list.analyzed.units",
+#'   "clinical_results.outcome_list.outcome",
+#'   "study_design_info.allocation",
+#'   # euctr - typical results fields
+#'   "trialInformation.fullTitle",
+#'   "subjectDisposition.recruitmentDetails",
+#'   "baselineCharacteristics.baselineReportingGroups.baselineReportingGroup",
+#'   "endPoints.endPoint",
+#'   "trialChanges.hasGlobalInterruptions",
+#'   "subjectAnalysisSets",
+#'   "adverseEvents.seriousAdverseEvents.seriousAdverseEvent"
+#'   ), con = dbc
+#' )
+#' # convert to long
+#' reslong <- ctrdata::dfTrials2Long(
+#'   df = df
+#' )
+#' # get values for endpoint of interest, duration of response
+#' ctrdata::dfValue2Name(
+#'   df = df,
+#'   valuename = paste0(
+#'     "endPoints.endPoint.*armReportingGroup.tendencyValues.tendencyValue.value|",
+#'     "clinical_results.*category.measurement_list.measurement.value|",
+#'     "clinical_results.*outcome.measure.units|endPoints.endPoint.unit"
+#'   ),
+#'   wherename = "clinical_results.*outcome.measure.title|endPoints.endPoint.title",
+#'   wherevalue = "duration of response"
+#' )
+#' }
+dfName2Value <- function(df, valuename = "",
+                         wherename = "", wherevalue = "") {
+
+  # check parameters
+  if (valuename == "") {
+    stop("'valuename' must be specified.",
+         call. = FALSE)
+  }
+  if (!identical(
+    names(df),
+    c("trial_id", "main_id",
+      "sub_id", "name", "value"))) {
+    stop("'df' does not seem to come from dfTrials2Long()",
+         call. = FALSE)
+  }
+
+  # if now where... are specified, just
+  # return the rows where name corresponds
+  # to valuename
+  if (wherename == "" & wherevalue == "") {
+
+    # get relevant rows
+    df <- df[grepl(valuename, df[["name"]]), ]
+
+    # value column is character
+    # try to convert it to numeric
+    out <- suppressWarnings(
+      as.numeric(df[["value"]])
+    )
+    # use if converted ok
+    if (all(is.na(out) == is.na(df[["value"]]))) {
+      df["value"] <- out
+    }
+
+    # output
+    return(df)
+
+  } # otherwise continue
+
+  # get indices per trial
+  index.names <- which(
+    grepl(wherename, df[["name"]],
+          perl = TRUE,
+          ignore.case = TRUE) &
+      grepl(wherevalue, df[["value"]],
+            perl = TRUE,
+            ignore.case = TRUE))
+
+  # get trial ids, main_id, sub_id
+  index.cases <-
+    df[index.names, c("trial_id", "main_id", "sub_id")]
+
+  # get output
+  out <- lapply(
+    seq_len(nrow(index.cases)),
+    function(i) {
+
+      tmp <- intersect(
+        which(df["trial_id"] == index.cases[i, "trial_id"]),
+        which(df["main_id"] == index.cases[i, "main_id"])
+      )
+      if (!is.na(index.cases[i, "sub_id"])) {
+        tmp <- intersect(
+          tmp,
+          which(df["sub_id"] == index.cases[i, "sub_id"])
+        )
+      }
+      df <- df[tmp, ]
+      tmp <- which(grepl(
+        valuename, df[ , "name"],
+        perl = FALSE,
+        ignore.case = TRUE)
+      )
+      if (length(tmp)) df[tmp, ]
+    }
+  )
+
+  # bind into data frame
+  out <- do.call(
+    rbind,
+    out
+  )
+
+  # value column is character
+  # try to convert it to numeric
+  tmp <- suppressWarnings(
+    as.numeric(out[["value"]])
+  )
+  # use if converted ok
+  if (all(is.na(tmp) == is.na(out[["value"]]))) {
+    out["value"] <- tmp
+  }
+
+  # return
+  return(out)
+
+} # end dfName2Value
+
+
+#' Extract trial information into long format
+#'
+#' The function works with procotol- and results-
+#' related information. It converts lists and other
+#' values into individual rows of a long data frame.
+#' From the resulting data frame, values of interest
+#' can then be selected (e.g. select and outcome
+#' and its analysis by number of the measure which
+#' has "Hazard Ratio" in its name).
+#'
+#' @param df Data frame with colums including
+#'  the trial identifier (\code{_id}) and
+#'  one or more variables as obtained from
+#'  \link{dbGetFieldsIntoDf}
+#'
+#' @return A data frame with the five columns:
+#'  trial_id, main_id, sub_id, name, value
+#'
+#' @export
+#'
+#' @examples
+#' \dontrun{
+#' db <- nodbi::src_sqlite(
+#'   collection = "my_collection"
+#' )
+#' df <- dbGetFieldsIntoDf(
+#'   fields = c(
+#'     "clinical_results.outcome_list.outcome"),
+#'   con = db
+#' )
+#' dfTrials2Long(
+#'   df = df
+#' )
+#' }
+dfTrials2Long <- function(df) {
+
+  # check parameters
+
+  # check if _id present
+  if (!any("_id" == names(df))) stop(
+    "Missing _id column / variable in parameter 'df'.",
+    call. = FALSE
+  )
+
+  # do by endpoint
+  col2df <- function(atrialcol) {
+
+    # the full column is needed in atrialcol
+    # to extract the name of the column
+    atcname <- paste0("", names(atrialcol))
+
+    # atrialcol is a single cell of a
+    # larger data.frame and this
+    # extracts the element in this cell
+    atrialcol <- atrialcol[[1]]
+
+    # if not inheriting from data.frame,
+    # change into one so that row indexing
+    # in lapply below will work
+    if (!inherits(atrialcol, "data.frame")) {
+      atrialcol <- data.frame(
+        atrialcol,
+        check.names = FALSE,
+        stringsAsFactors = FALSE)
+    }
+
+    # for each endpoint / measure,
+    # expand the lists into long
+    # format by unlisting
+    measures <- lapply(
+
+      seq_len(nrow(atrialcol)),
+      function(r) {
+
+        # get rows of data
+        measure <- atrialcol[r, ]
+
+        # fully unlist
+        measure <- unlist(
+          measure,
+          recursive = TRUE,
+          use.names = TRUE)
+
+        # in case the column is empty
+        if (is.null(measure)) {
+          data.frame(
+            main_id = NA,
+            sub_id = NA,
+            name = atcname,
+            value = NA,
+            stringsAsFactors = FALSE
+          )
+        } else {
+          # return result
+          data.frame(
+            main_id = r,
+            sub_id = NA,
+            name = paste0(atcname, ".", names(measure)),
+            value = measure,
+            stringsAsFactors = FALSE
+          )
+        }
+
+      })
+
+    # concatenate into long format
+    measures <- do.call(
+      rbind,
+      measures
+    )
+
+    # format names
+    measures["sub_id"] <- suppressWarnings(
+      as.integer(
+        sub("^.*?([0-9]+)$", "\\1", measures$name)
+      ))
+    measures["name"] <- sub("[0-9]+$", "", measures$name)
+    measures["name"] <- sub("@attributes[.]", "", measures$name)
+
+    # output
+    row.names(measures) <- NULL
+    measures
+
+  }
+
+  # for each trial
+  trials2long <- function(severaltrials) {
+
+    # total number of trials
+    numtrials <- nrow(severaltrials)
+
+    # iterate over trials
+    trials <- lapply(
+      seq_len(nrow(severaltrials)),
+      function(r) {
+
+        # r-th trial in works
+        message("Trial ", r, " / ", numtrials, "\r", appendLF = FALSE)
+
+        # get trial
+        atrial = severaltrials[r, ]
+
+        # iterate over columns
+        cols <- lapply(
+          seq_len(ncol(severaltrials[r, ])),
+          function(c) {
+
+            # c-th column in the works
+            # message(c, " ", appendLF = FALSE)
+
+            # convert into long format
+            switch(
+              typeof(atrial[[c]]),
+              "character" = data.frame(
+                main_id = NA,
+                sub_id = NA,
+                name = names(atrial)[c],
+                value = atrial[[c]],
+                stringsAsFactors = FALSE
+              ),
+              "list" = col2df(atrialcol = atrial[c])
+            )
+          }
+        )
+
+        # concatenate
+        out <- do.call(
+          rbind,
+          cols
+        )
+
+        # format trial
+        data.frame(
+          trial_id = out[out[["name"]] == "_id", "value"],
+          out[out[["name"]] != "_id", ],
+          stringsAsFactors = FALSE
+        )
+
+      })
+
+    # concatenate into long format
+    do.call(
+      rbind,
+      trials
+    )
+
+  }
+
+  # output
+  return(
+    trials2long(
+      severaltrials = df
+    )
+  )
+
+} # end dfTrials2Long
+
+
 #' Extract named element(s) from list(s) into long-format
 #' data frame
 #'
@@ -1391,6 +1751,9 @@ dfListExtractKey <- function(
     list(c("endPoints.endPoint", "^title")
     )) {
 
+  # deprecate
+  .Deprecated("dfName2Value")
+
   # check
   if (!("_id" %in% names(df))) {
     stop("Data frame 'df' lacks '_id' column.",
@@ -1462,7 +1825,8 @@ dfListExtractKey <- function(
 
   # return
   out
-}
+
+} # end dfListExtractKey
 
 
 #' Merge two variables into one, optionally map values to new levels
@@ -1526,10 +1890,38 @@ dfMergeTwoVariablesRelevel <- function(
   # find variables in data frame and merge
   tmp <- match(colnames, names(df))
   df <- df[, tmp]
-  df[, 1] <- ifelse(is.na(tt <- df[, 1]), "", tt)
-  df[, 2] <- ifelse(is.na(tt <- df[, 2]), "", tt)
-  tmp <- paste0(df[, 1], df[, 2])
 
+  # # bind as ...
+  if (class(df[, 1]) == class(df[, 2]) &&
+      class(df[, 1]) != "character") {
+    # values, with first having
+    # priority over the second
+    tmp <- ifelse(is.na(tt <- df[, 1]), df[, 2], df[, 1])
+    if (nrow(df)) {
+      warning("Some rows had values for both columns, ",
+              "used ", colnames[1],
+              noBreaks. = TRUE, immediate. = TRUE)
+    }
+  } else {
+    # strings, concatenated
+    tmp <- paste0(
+      ifelse(is.na(tt <- as.character(df[, 1])), "", tt),
+      ifelse(is.na(tt <- as.character(df[, 2])), "", tt))
+  }
+
+  # # bind as strings, concatenated
+  # tmp <- paste0(
+  #   ifelse(is.na(tt <- as.character(df[, 1])), "", tt),
+  #   ifelse(is.na(tt <- as.character(df[, 2])), "", tt))
+
+  # and then type where possible
+  if (class(df[, 1]) == class(df[, 2]) &&
+      class(df[, 1]) != "character") {
+    mode(tmp) <- mode(df[, 1])
+    class(tmp) <- class(df[, 1])
+  }
+
+  # relevel is specified
   if (!is.null(levelslist)) {
 
     # check
