@@ -23,7 +23,7 @@ if (.Platform$OS.type != "windows") {
 if (.Platform$OS.type == "windows") {
   expect_message(
     installCygwinWindowsDoInstall(),
-  "cygwin is already installed")
+    "cygwin is already installed")
 }
 
 
@@ -31,37 +31,42 @@ if (.Platform$OS.type == "windows") {
 
 # test
 expect_error(
-  dfMergeTwoVariablesRelevel(list("var1", "var2")),
+  suppressWarnings(
+    dfMergeTwoVariablesRelevel(list("var1", "var2"))),
   "Need a data frame as input.")
 
 # test
 expect_message(
-  dfMergeTwoVariablesRelevel(
-    df = df,
-    colnames = c("var1", "var2")),
-  "Unique values returned: 12, 23, 34")
+  suppressWarnings(
+    dfMergeTwoVariablesRelevel(
+      df = df,
+      colnames = c("var1", "var2"))),
+  "Unique values returned: 1, 2, 3")
 
 # test
 expect_true(
-  "character" %in% class(
-    suppressMessages(
-      dfMergeTwoVariablesRelevel(
-        df = df,
-        colnames = c("var1", "var2")))))
+  "integer" %in% class(
+    suppressWarnings(
+      suppressMessages(
+        dfMergeTwoVariablesRelevel(
+          df = df,
+          colnames = c("var1", "var2"))))))
 
 # test
 expect_message(
-  dfMergeTwoVariablesRelevel(
-    df = df,
-    colnames = c("var1", "var2"),
-    levelslist = statusvalues),
-  "Unique values returned: Firstvalues, Lastvalue")
+  suppressWarnings(
+    dfMergeTwoVariablesRelevel(
+      df = df,
+      colnames = c("var1", "var2"),
+      levelslist = statusvalues)),
+  "Unique values returned: 1, 2, 3")
 
 # test
 expect_error(
-  dfMergeTwoVariablesRelevel(
-    df = df,
-    colnames = 1:3),
+  suppressWarnings(
+    dfMergeTwoVariablesRelevel(
+      df = df,
+      colnames = 1:3)),
   "Please provide exactly two column names.")
 
 # test
@@ -72,11 +77,19 @@ expect_warning(
   "Parameter varnames is deprecated, use colnames instead")
 
 # test
-expect_error(
+expect_warning(
   dfMergeTwoVariablesRelevel(
     df = df,
-    colnames = c("var1", "var2"),
-    levelslist = 1:2),
+    varnames = c("var1", "var2")),
+  "Some rows had values for both columns")
+
+# test
+expect_error(
+  suppressWarnings(
+    dfMergeTwoVariablesRelevel(
+      df = df,
+      colnames = c("var1", "var2"),
+      levelslist = 1:2)),
   "Need list for parameter 'levelslist'")
 
 
